//
//  LSClientMessageDelegate.h
//  Lightstreamer client for iOS UCA
//

#import <Foundation/Foundation.h>


@class LSLightstreamerClient;


/**
 Protocol to be implemented to receive <[LSLightstreamerClient sendMessage:withSequence:timeout:delegate:enqueueWhileDisconnected:]> events reporting a message
 processing outcome.

 Events for these delegates are dispatched by a different thread than the one that generates them. All the notifications for a single <LSLightstreamerClient>,
 including notifications to <LSClientDelegate>, <LSSubscriptionDelegate> and LSClientMessageDelegate, will be dispatched by the same thread. Only one event
 per message is fired on this delegate.
 */
@protocol LSClientMessageDelegate <NSObject>


@optional

/**
 @brief Event handler that is called by Lightstreamer when any notifications of the processing outcome of the related message haven't been received yet and
 can no longer be received.

 Typically, this happens after the session has been closed. In this case, the client has no way of knowing the processing outcome and any outcome is possible.

 @param client The <LSLightstreamerClient> instance.

 @param originalMessage The message to which this notification is related.

 @param sentOnNetwork YES if the message was sent on the network, NO otherwise. Even if the flag is YES, it is not possible to infer whether the message
 actually reached the Lightstreamer Server or not.
 */
- (void) client:(nonnull LSLightstreamerClient *)client didAbortMessage:(nonnull NSString *)originalMessage sentOnNetwork:(BOOL)sentOnNetwork;

/**
 @brief Event handler that is called by Lightstreamer when the related message has been processed by the Server but the expected processing outcome could
 not be achieved for any reason.

 @param client The <LSLightstreamerClient> instance.

 @param originalMessage The message to which this notification is related.

 @param code The error code sent by the Server. The code value is dependent on the specific Metadata Adapter implementation.

 @param error The description of the error sent by the Server.
 */
- (void) client:(nonnull LSLightstreamerClient *)client didDenyMessage:(nonnull NSString *)originalMessage withCode:(NSInteger)code error:(nonnull NSString *)error;

/**
 @brief Event handler that is called by Lightstreamer to notify that the related message has been discarded by the Server.

 This means that the message has not reached the Metadata Adapter and the message next in the sequence is considered enabled for processing.

 @param client The <LSLightstreamerClient> instance.

 @param originalMessage The message to which this notification is related.
 */
- (void) client:(nonnull LSLightstreamerClient *)client  didDiscardMessage:(nonnull NSString *)originalMessage;

/**
 @brief Event handler that is called by Lightstreamer when the related message has been processed by the Server but the processing has failed for any reason.

 The level of completion of the processing by the Metadata Adapter cannot be determined.

 @param client The <LSLightstreamerClient> instance.

 @param originalMessage The message to which this notification is related.
 */
- (void) client:(nonnull LSLightstreamerClient *)client  didFailMessage:(nonnull NSString *)originalMessage;

/**
 @brief Event handler that is called by Lightstreamer when the related message has been processed by the Server with success.

 @param client The <LSLightstreamerClient> instance.

 @param originalMessage The message to which this notification is related.
 */
- (void) client:(nonnull LSLightstreamerClient *)client  didProcessMessage:(nonnull NSString *)originalMessage;


@end

